#import <Foundation/Foundation.h>

@class B4IArray;
@class B4I;

//~version:1.01
//~DependsOn:libz.dylib
//~Shortname:RandomAccessFile
@interface B4IRandomAccessFile : NSObject
/**
* Returns the current file position. The position is updated automatically after each operation.
*/
@property (nonatomic) long long CurrentPosition;
/**
* Returns the total size.
*/
@property (nonatomic, readonly)long long Size;

/**
* Opens the specified file in big endian mode.
*/
- (void)Initialize:(NSString *)Dir :(NSString *)FileName :(BOOL)ReadOnly;

/**
* Opens the specified file with the specified endianess mode.
*/
- (void)Initialize2:(NSString *)Dir :(NSString *)FileName :(BOOL)ReadOnly :(BOOL)LittleEndian;

/**
* Treats the array of bytes as a random access file with a constant size.
*/
- (void)Initialize3:(B4IArray */*byte,1*/)Buffer :(BOOL)LittleEndian;

/**
* Closes the file.
*/
- (void)Close;

/**
* Writes any cached data to the file.
*/
- (void)Flush;

/**
* Reads a Short value (2 bytes). The position is updated automatically.
*/
- (short)ReadShort:(long long)Position;

/**
* Reads an Int value (4 bytes). The position is updated automatically.
*/
- (int)ReadInt:(long long)Position;
//Reads a Long value (8 bytes). The position is updated automatically.
- (long long)ReadLong:(long long)Position;
//Reads a Float value (4 bytes). The position is updated automatically.
- (float)ReadFloat:(long long)Position;
//Reads a Double value (8 bytes). The position is updated automatically.
- (double)ReadDouble:(long long)Position;

/**
* Reads the bytes from Position to Position + Length (exclusive) and writes them to the given array starting from StartOffset.
* The position is updated automatically.
*/
- (void)ReadBytes:(B4IArray */*byte,1*/)Buffer :(int)StartOffset :(int)Length :(long long)Position;
//Writes the bytes from the array to the file.  The position is updated automatically.
- (void)WriteBytes:(B4IArray */*byte,1*/)Buffer :(int)StartOffset :(int)Length :(long long)Position;
//Writes a Short value (2 bytes). The position is updated automatically.
- (void)WriteShort:(short)Value :(long long)Position;
//Writes a single Byte. The position is updated automatically.
- (void)WriteByte:(unsigned char)Value :(long long)Position;
//Writes an Int value (4 bytes). The position is updated automatically.
- (void)WriteInt:(int)Value :(long long)Position;
//Writes a Long value (8 bytes). The position is updated automatically.
- (void)WriteLong:(long long)Value :(long long)Position;
//Writes Double value (8 bytes). The position is updated automatically.
- (void)WriteDouble:(double)Value :(long long)Position;
//Writes a Float value (4 bytes). The position is updated automatically.
- (void)WriteFloat:(float)Value :(long long)Position;

/**
* Writes the given object to the file.
* This method is capable of writing the following types: Lists, Arrays, Maps, Strings, primitive types and user types.
* Combinations of these types are also supported.
*/
- (void)WriteB4XObject:(NSObject *)Object :(long long)Position;

/**
* Reads an object previously saved with WriteB4XObject. See WriteB4XObject for the supported types.
*/
- (NSObject *)ReadB4XObject:(long long)Position;

//Reads a single byte. The position is updated automatically.
- (unsigned char)ReadUnsignedByte:(long long)Position;
@end
//~shortname:ByteConverter
@interface B4IByteConverter : NSObject
/**
* Gets or sets the converter endianess. The default value is False (big endian).
*/
@property (nonatomic)BOOL LittleEndian;

//Converts an array of Bytes to an array of Ints. Each 4 bytes are converted to an Int.
- (B4IArray */*int,1*/)IntsFromBytes:(B4IArray */*byte,1*/)Bytes;
//Converts an array of Shorts to an array of Bytes. Each Short is converted to 2 bytes.
- (B4IArray */*byte,1*/)ShortsToBytes:(B4IArray */*short,1*/)Shorts;
//Converts an array of Longs to an array of Bytes. Each Long is converted to 8 bytes.
- (B4IArray */*byte,1*/)LongsToBytes:(B4IArray */*long,1*/)Longs;
//Converts an array of Bytes to an array of Longs. Each 8 bytes are converted to a Long.
- (B4IArray */*long,1*/)LongsFromBytes:(B4IArray */*byte,1*/)Bytes;
//Converts an array of Ints to an array of Bytes. Each Int is converted to 4 bytes.
- (B4IArray */*byte,1*/)IntsToBytes:(B4IArray */*int,1*/)Ints;
//Converts an array of Bytes to an array of Shorts. Each 2 bytes are converted to a Short.
- (B4IArray */*short,1*/)ShortsFromBytes:(B4IArray */*byte,1*/)Bytes;
//Converts an array of Bytes to an array of Doubles. Each 8 bytes are converted to a Double.
- (B4IArray */*double,1*/)DoublesFromBytes:(B4IArray */*byte,1*/)Bytes;
//Converts an array of Doubles to an array of Bytes. Each Double is converted to 8 bytes.
- (B4IArray */*byte,1*/)DoublesToBytes:(B4IArray */*double,1*/)Doubles;
//Converts the bytes to a string. Similar to BytesToString keyword.
- (NSString *)StringFromBytes:(B4IArray */*byte,1*/)Bytes :(NSString *)Encoding;
//Converts the string to an array of bytes.
- (B4IArray */*byte,1*/)StringToBytes:(NSString*)Str :(NSString *)Encoding;
//Converts an array of Bytes to an array of Floats. Each 4 bytes are converted to a Float.
- (B4IArray */*float,1*/)FloatsFromBytes:(B4IArray */*byte,1*/)Bytes;
//Converts an array of Floats to an array of Bytes. Each Float is converted to 4 bytes.
- (B4IArray */*byte,1*/)FloatsToBytes:(B4IArray */*float,1*/)Doubles;
//Returns the hex representation of the given bytes.
- (NSString *)HexFromBytes:(B4IArray */*byte,1*/)Bytes;
//Parses a hex string to an array of bytes.
- (B4IArray */*byte,1*/)HexToBytes:(NSString *)Hex;
/**
* Copies data from the source array to the destination array.
* Src - The source array.
* SrcOffset - The position of the first byte that will be copied.
* Dest - The destination array.
* DestOffset - The position in the destination array of the first byte.
* Count - Number of bytes to copy.
*/
- (void)ArrayCopy:(B4IArray */*byte,1*/)Src :(int)SrcOffset :(B4IArray */*byte,1*/)Dest :(int)DestOffset :(int)Count;
//Returns an array of strings with the supported encodings.
- (B4IArray */*NSString,1*/)SupportedEncodings;
@end

//~shortname:CompressedStreams
@interface B4ICompressedStreams:NSObject
/**
* Compresses the Data array using zlib or gzip compression. Returns the compressed data as an array of bytes.
*Data - The data to compress.
* CompressMethod - "zlib" or "gzip"
*/
- (B4IArray */*byte,1*/)CompressBytes:(B4IArray */*byte,1*/)Data :(NSString *)CompressMethod;

/**
* Decompresses the data using zlib or gzip compression. Returns the uncompressed data as an array of bytes.
* CompressedData - The data to decompress.
* CompressMethod - "zlib" or "gzip"
*/
- (B4IArray */*byte,1*/)DecompressBytes:(B4IArray */*byte,1*/)CompressedData :(NSString *)CompressMethod;
@end

//~shortname:AsyncStreams
//~Event:NewData (Buffer() As Byte)
//~Event:Error
//~Event:Terminated
@interface B4IAsyncStreams :NSObject
/**
* Initializes AsyncStreams in standard mode.
*/
- (void)Initialize:(B4I *)bi1 :(NSInputStream *)In :(NSOutputStream *)Out :(NSString *)EventName;

/**
* Initializes AsyncStreams in prefix mode. This mode can only be used if both sides of the connection adhere to the "prefix" protocol.
* In / Out - The monitored streams.
* BigEndian - The prefix endianess.
*/
- (void)InitializePrefix:(B4I *)bi1 :(NSInputStream *)In :(BOOL)BigEndian :(NSOutputStream *)Out :(NSString *)EventName;

/**
* Asynchronously writes all the data in the array.
*/
- (BOOL)Write:(B4IArray */*byte,1*/)Buffer;

/**
* Asynchronously writes the data.
*Buffer - The array holding the data.
* Start - The position in the array of the first byte that will be written.
* Length - Number of bytes to write.
*/
- (BOOL)Write2:(B4IArray */*byte,1*/)Buffer :(int)Start :(int)Length;

/**
* Closes the monitored streams.
*/
- (void)Close;

- (BOOL)IsInitialized;
@end
//~hide
@interface B4ISerializator : NSObject

- (B4IArray *)WriteObject:(NSObject *)object :(BOOL)compressed;

- (NSObject *)ReadObject:(B4IArray *)arr :(BOOL)compressed;
@end


